require 'forwardable'

module Steam
  module Id
    # Represents a SteamID holds the various parts of the Steam Id an internal
    # 64bit datastructure
    class Base
      extend Forwardable

      # Delegate some getter methods to the internal data struct
      def_delegators :@struct, :account_id, :account_type, :instance, :universe

      # Delegate some setter methods to the internal data struct
      def_delegators :@struct, :account_id=,
                     :account_type=, :instance=, :universe=

      attr_reader :struct

      # Create a new Steam ID
      def initialize(struct = Struct.new)
        @struct = struct
      end

      # Return the Steam Id in string format. By default the Steam2 format
      # is returned
      #
      # @param klass [Printer] a steam id printer
      # @return [String] the steam id in the desired format
      def to_s(klass = Steam2Printer)
        printer ||= klass.new(self)
        printer.print
      end

      # Return the Steam Id in int format.
      #
      # @return [Integer] the steam id in int format
      def to_i
        @struct.to_i
      end

      # Returns true if the Steam id is valid
      #
      # @return [Bool] true if valid, otherwise false
      def valid?
        valid_universe? && valid_account?
      end

      private

      # @api private
      def valid_account?
        return false if account_type <= EAccountType::INVALID ||
                        account_type >= EAccountType::MAX

        if account_type == EAccountType::INDIVIDUAL
          return false if account_id.zero?
        end

        true
      end

      # @api private
      def valid_universe?
        return false if universe <= EUniverse::INVALID ||
                        universe >= EUniverse::MAX
        true
      end
    end
  end
end
